@@ -10,11 +10,21 @@ use Carp;
 with 'Lemonldap::NG::Portal::Lib::2fDevices';
 use Lemonldap::NG::Common::Util qw/display2F/;
 
-our $VERSION = '2.17.0';
+our $VERSION = '2.18.0';
 
 has rp_id    => ( is => 'rw', lazy => 1, builder => "_build_rp_id" );
 has origin   => ( is => 'rw', lazy => 1, builder => "_build_origin" );
 has verifier => ( is => 'rw', lazy => 1, builder => "_build_verifier" );
+has trust_anchors => (
+    is      => 'rw',
+    lazy    => 1,
+    isa     => "ArrayRef",
+    builder => "_build_trust_anchors"
+);
+
+sub _build_trust_anchors {
+    return [];
+}
 
 sub _build_verifier {
     my $self = shift;
@@ -102,12 +112,22 @@ sub validateCredential {
         $req->headers->header('Sec-Provided-Token-Binding-ID') )
       : '';
 
+    # If Authen::WebAuthn is too old, we can't check attestation
+    my $attestation = $registration_options->{attestation} || "none";
+    if ($attestation ne "none" and $Authen::WebAuthn::VERSION < 0.002) {
+        croak("Authen::WebAuthn version is too old, cannot validate attestation");
+    }
+
+
     return $self->verifier->validate_registration(
         challenge_b64          => $challenge_b64,
         requested_uv           => $requested_uv,
         client_data_json_b64   => $client_data_json_b64,
         attestation_object_b64 => $attestation_object_b64,
-        token_binding_id_b64   => $token_binding_id_b64
+        token_binding_id_b64   => $token_binding_id_b64,
+        trust_anchors          => $self->trust_anchors,
+        ($attestation ne "none" ? (
+            allowed_attestation_types => [ "Basic" ] ) : ()),
     );
 }
 
