@@ -68,7 +68,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_PASSWORD_OK
 );
 
-our $VERSION = '2.17.0';
+our $VERSION = '2.17.2';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
@@ -786,9 +786,8 @@ sub getUser {
 sub myApplications {
     my ( $self, $req ) = @_;
     my $basePath = $self->conf->{portal};
-    $basePath =~ s#/*$#/#;
+    $basePath =~ s#/*$##;
     $basePath .= $self->p->{staticPrefix} . '/common/apps/';
-    $basePath =~ s#//+#/#;
     my @appslist = map {
         my @apps = map {
             {
